@@ -17,6 +17,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <atk/atk.h>
 #include <gtk/gtk.h>
@@ -43,6 +44,7 @@ static AtkObject* gail_get_accessible_for_widget (GtkWidget    *widget,
                                                   gboolean     *transient);
 static void     gail_finish_select       (GtkWidget            *widget);
 static void     gail_map_cb              (GtkWidget            *widget);
+static void     gail_show_cb             (GtkWidget            *widget);
 static gint     gail_focus_idle_handler  (gpointer             data);
 static void     gail_focus_notify        (GtkWidget            *widget);
 static void     gail_focus_notify_when_idle (GtkWidget            *widget);
@@ -127,12 +129,6 @@ gail_get_accessible_for_widget (GtkWidget *widget,
           g_object_unref (obj);
         }
     }
-  else if (GTK_IS_TREE_VIEW (widget)) 
-    {
-      obj = gail_tree_view_ref_focus_cell (GTK_TREE_VIEW (widget));
-      if (obj)
-        *transient = TRUE;
-    }
   else if (GNOME_IS_CANVAS (widget)) 
     {
       GnomeCanvas *canvas;
@@ -280,6 +276,49 @@ gail_select_watcher (GSignalInvocationHint *ihint,
 static void
 gail_finish_select (GtkWidget *widget)
 {
+  if (GTK_IS_MENU_ITEM (widget))
+    {
+      GtkMenuItem* menu_item;
+
+      menu_item = GTK_MENU_ITEM (widget);
+      if (menu_item->submenu &&
+          !GTK_WIDGET_VISIBLE (menu_item->submenu))
+        {
+          /*
+           * If the submenu is not visble, wait until it is before
+           * reporting focus on the menu item.
+           */
+          gulong handler_id;
+
+          handler_id = g_signal_handler_find (menu_item->submenu,
+                                              G_SIGNAL_MATCH_FUNC,
+                                              g_signal_lookup ("show",
+                                                               GTK_TYPE_WINDOW),
+                                              0,
+                                              NULL,
+                                              (gpointer) gail_show_cb,
+                                              NULL); 
+          if (!handler_id)
+            g_signal_connect (menu_item->submenu, "show",
+                              G_CALLBACK (gail_show_cb),
+                              NULL);
+
+          /*
+           * If we are waiting to report focus on a menubar or a menu item
+           * because of a previous deselect, cancel it.
+           */
+          if (focus_notify_handler &&
+              next_focus_widget &&
+              (GTK_IS_MENU_BAR (next_focus_widget) ||
+               GTK_IS_MENU_ITEM (next_focus_widget)))
+            {
+              g_source_remove (focus_notify_handler);
+              g_object_remove_weak_pointer (G_OBJECT (next_focus_widget), (gpointer *)&next_focus_widget);
+              focus_notify_handler = 0;
+            }
+          return;
+        }
+    } 
   /*
    * If previously focused widget is not a GtkMenuItem or a GtkMenu,
    * keep track of it so we can return to it after menubar is deactivated
@@ -303,6 +342,17 @@ gail_map_cb (GtkWidget *widget)
   gail_finish_select (widget);
 }
 
+static void
+gail_show_cb (GtkWidget *widget)
+{
+  if (GTK_IS_MENU (widget))
+    {
+      if (GTK_MENU (widget)->parent_menu_item)
+        gail_finish_select (GTK_MENU (widget)->parent_menu_item);
+    }
+}
+
+
 static gboolean
 gail_deselect_watcher (GSignalInvocationHint *ihint,
                        guint                  n_param_values,
@@ -442,7 +492,7 @@ gail_focus_notify_when_idle (GtkWidget *widget)
     {
       if (widget)
         {
-          gtk_idle_remove (focus_notify_handler);
+          g_source_remove (focus_notify_handler);
           if (next_focus_widget)
             g_object_remove_weak_pointer (G_OBJECT (next_focus_widget), (gpointer *)&next_focus_widget);
         }
@@ -458,7 +508,7 @@ gail_focus_notify_when_idle (GtkWidget *widget)
       next_focus_widget = widget;
       g_object_add_weak_pointer (G_OBJECT (next_focus_widget), (gpointer *)&next_focus_widget);
     }
-  focus_notify_handler = gtk_idle_add (gail_focus_idle_handler, widget);
+  focus_notify_handler = g_idle_add (gail_focus_idle_handler, widget);
 }
 
 static gboolean
@@ -562,7 +612,7 @@ gail_accessibility_module_init (void)
     }
   gail_initialized = TRUE;
 
-  g_print ("GTK Accessibility Module initialized\n");
+  fprintf (stderr, "GTK Accessibility Module initialized\n");
 
   GAIL_WIDGET_SET_FACTORY (GTK_TYPE_WIDGET, gail_widget);
   GAIL_WIDGET_SET_FACTORY (GTK_TYPE_CONTAINER, gail_container);
@@ -611,6 +661,7 @@ gail_accessibility_module_init (void)
   GAIL_WIDGET_SET_FACTORY (GNOME_TYPE_CANVAS_RICH_TEXT, gail_canvas_text);
   GAIL_WIDGET_SET_FACTORY (GNOME_TYPE_CANVAS_WIDGET, gail_canvas_widget);
   GAIL_WIDGET_SET_FACTORY (GNOME_TYPE_CANVAS_ITEM, gail_canvas_item);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_OBJECT, gail_object);
 
   atk_focus_tracker_init (gail_focus_tracker_init);
 
@@ -647,7 +698,7 @@ gnome_accessibility_module_shutdown (void)
     }
   gail_initialized = FALSE;
 
-  g_print("Gtk Accessibilty Module shutdown\n");
+  fprintf (stderr, "GTK Accessibility Module shutdown\n");
 
   /* FIXME: de-register the factory types so we can unload ? */
 }
