@@ -33,6 +33,8 @@
 #include <gail/gailmenushell.h>
 #include <gail/gailmenuitem.h>
 #include <gail/gailnotebook.h>
+#include <gail/gailobject.h>
+#include <gail/gailobjectfactory.h>
 #include <gail/gailoptionmenu.h>
 #include <gail/gailpaned.h>
 #include <gail/gailpixmap.h>
