@@ -1,5 +1,5 @@
 /* GAIL - The GNOME Accessibility Implementation Library
- * Copyright 2001 Sun Microsystems Inc.
+ * Copyright 2001, 2002, 2003 Sun Microsystems Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -131,6 +131,8 @@ static void
 gail_combo_object_init (GailCombo      *combo)
 {
   combo->press_description = NULL;
+  combo->old_selection = NULL;
+  combo->deselect_idle_handler = NULL;
 }
 
 AtkObject* 
@@ -146,8 +148,6 @@ gail_combo_new (GtkWidget *widget)
   accessible = ATK_OBJECT (object);
   atk_object_initialize (accessible, widget);
 
-  accessible->role = ATK_ROLE_COMBO_BOX;
-
   return accessible;
 }
 
@@ -156,30 +156,79 @@ gail_combo_real_initialize (AtkObject *obj,
                             gpointer  data)
 {
   GtkCombo *combo;
-  GtkWidget *list, *entry;
-  AtkObject *child_accessible;
+  GtkList *list;
+  GList *slist; 
+  GailCombo *gail_combo;
 
   ATK_OBJECT_CLASS (parent_class)->initialize (obj, data);
 
   combo = GTK_COMBO (data);
-  list = combo->list;
-  entry = combo->entry;
 
-  g_signal_connect (list,
+  list = GTK_LIST (combo->list);
+  slist = list->selection;
+
+  gail_combo = GAIL_COMBO (obj);
+  if (slist && slist->data)
+    {
+      gail_combo->old_selection = slist->data;
+    }
+  g_signal_connect (combo->list,
                     "selection_changed",
                     G_CALLBACK (gail_combo_selection_changed_gtk),
-                    obj);
-  child_accessible = gtk_widget_get_accessible (entry);
-  atk_object_set_parent (child_accessible, obj);
-  child_accessible = gtk_widget_get_accessible (list);
-  atk_object_set_parent (child_accessible, obj);
+                    data);
+  atk_object_set_parent (gtk_widget_get_accessible (combo->entry), obj);
+  atk_object_set_parent (gtk_widget_get_accessible (combo->popup), obj);
+
+  obj->role = ATK_ROLE_COMBO_BOX;
+}
+
+static gboolean
+notify_deselect (gpointer data)
+{
+  GailCombo *combo;
+
+  combo = GAIL_COMBO (data);
+
+  combo->old_selection = NULL;
+  g_signal_emit_by_name (data, "selection_changed");
+  return FALSE;
 }
 
 static void
 gail_combo_selection_changed_gtk (GtkWidget      *widget,
                                   gpointer       data)
 {
-  g_signal_emit_by_name (data, "selection_changed");
+  GtkCombo *combo;
+  GtkList *list;
+  GList *slist; 
+  AtkObject *obj;
+  GailCombo *gail_combo;
+
+  combo = GTK_COMBO (data);
+  list = GTK_LIST (combo->list);
+  
+  slist = list->selection;
+
+  obj = gtk_widget_get_accessible (GTK_WIDGET (data));
+  gail_combo = GAIL_COMBO (obj);
+  if (slist && slist->data)
+    {
+      if (slist->data != gail_combo->old_selection)
+        {
+          gail_combo->old_selection = slist->data;
+          g_signal_emit_by_name (obj, "selection_changed");
+        }
+      if (gail_combo->deselect_idle_handler)
+        {
+          g_source_remove (gail_combo->deselect_idle_handler);
+          gail_combo->deselect_idle_handler = 0;       
+        }
+    }
+  else
+    {
+      if (gail_combo->deselect_idle_handler == 0)
+        gail_combo->deselect_idle_handler =  g_idle_add (notify_deselect, gail_combo);
+    }
 }
 
 static gboolean
@@ -237,7 +286,7 @@ gail_combo_ref_child (AtkObject *obj,
     return NULL;
 
   if (i == 0)
-    accessible = gtk_widget_get_accessible (GTK_COMBO (widget)->list);
+    accessible = gtk_widget_get_accessible (GTK_COMBO (widget)->popup);
   else if (_gail_combo_is_entry_editable (GTK_COMBO (widget)->entry))
     accessible = gtk_widget_get_accessible (GTK_COMBO (widget)->entry);
   else
@@ -282,7 +331,7 @@ gail_combo_do_action (AtkAction *action,
       if (combo->action_idle_handler)
         return FALSE;
 
-      combo->action_idle_handler = gtk_idle_add (idle_do_action, combo);
+      combo->action_idle_handler = g_idle_add (idle_do_action, combo);
       return TRUE;
     }
   else
@@ -337,7 +386,7 @@ idle_do_action (gpointer data)
 
       gtk_widget_event (action_widget, &tmp_event);
 
-      gtk_idle_add (_gail_combo_button_release, combo);
+      g_idle_add (_gail_combo_button_release, combo);
     }
     else
     {
@@ -347,7 +396,7 @@ idle_do_action (gpointer data)
       action_widget = combo->popwin;
     
       gtk_widget_event (action_widget, &tmp_event);
-      gtk_idle_add (_gail_combo_popup_release, combo);
+      g_idle_add (_gail_combo_popup_release, combo);
     }
   return FALSE;
 }
@@ -618,8 +667,13 @@ gail_combo_finalize (GObject            *object)
   g_free (combo->press_description);
   if (combo->action_idle_handler)
     {
-      gtk_idle_remove (combo->action_idle_handler);
+      g_source_remove (combo->action_idle_handler);
       combo->action_idle_handler = 0;
     }
+  if (combo->deselect_idle_handler)
+    {
+      g_source_remove (combo->deselect_idle_handler);
+      combo->deselect_idle_handler = 0;       
+    }
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
