@@ -1,5 +1,5 @@
 /* GAIL - The GNOME Accessibility Implementation Library
- * Copyright 2001 Sun Microsystems Inc.
+ * Copyright 2001, 2002, 2003 Sun Microsystems Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -127,6 +127,7 @@ static void       gail_entry_paste_received	   (GtkClipboard *clipboard,
 
 /* Callbacks */
 
+static void       gail_entry_notify_insert         (GailEntry            *entry);
 static void	  _gail_entry_insert_text_cb	   (GtkEntry     	 *entry,
                                                     gchar		 *arg1,
                                                     gint		 arg2,
@@ -272,6 +273,11 @@ gail_entry_real_initialize (AtkObject *obj,
 	G_CALLBACK (_gail_entry_delete_text_cb), NULL);
   g_signal_connect (data, "changed",
 	G_CALLBACK (_gail_entry_changed_cb), NULL);
+
+  if (entry->visible)
+    obj->role = ATK_ROLE_TEXT;
+  else
+    obj->role = ATK_ROLE_PASSWORD_TEXT;
 }
 
 static void
@@ -290,14 +296,8 @@ gail_entry_real_notify_gtk (GObject		*obj,
 
   if (strcmp (pspec->name, "cursor-position") == 0)
     {
-      if (entry->signal_name)
-        {
-          g_signal_emit_by_name (atk_obj, 
-                                 entry->signal_name,
-                                 entry->position,
-                                 entry->length);
-          entry->signal_name = NULL;
-        }
+      gail_entry_notify_insert (entry);
+
       if (check_for_selection_change (entry, gtk_entry))
         g_signal_emit_by_name (atk_obj, "text_selection_changed");
       /*
@@ -308,14 +308,8 @@ gail_entry_real_notify_gtk (GObject		*obj,
     }
   else if (strcmp (pspec->name, "selection-bound") == 0)
     {
-      if (entry->signal_name)
-        {
-          g_signal_emit_by_name (atk_obj, 
-                                 entry->signal_name,
-                                 entry->position,
-                                 entry->length);
-          entry->signal_name = NULL;
-        }
+      gail_entry_notify_insert (entry);
+
       if (check_for_selection_change (entry, gtk_entry))
         g_signal_emit_by_name (atk_obj, "text_selection_changed");
     }
@@ -387,7 +381,7 @@ gail_entry_finalize (GObject            *object)
   g_free (entry->activate_description);
   if (entry->action_idle_handler)
     {
-      gtk_idle_remove (entry->action_idle_handler);
+      g_source_remove (entry->action_idle_handler);
       entry->action_idle_handler = 0;
     }
   G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -406,11 +400,6 @@ gail_entry_new (GtkWidget *widget)
   accessible = ATK_OBJECT (object);
   atk_object_initialize (accessible, widget);
 
-  if (GTK_ENTRY (widget)->visible)
-    accessible->role = ATK_ROLE_TEXT;
-  else
-    accessible->role = ATK_ROLE_PASSWORD_TEXT;
-
   return accessible;
 }
 
@@ -1061,13 +1050,35 @@ gail_entry_paste_received (GtkClipboard *clipboard,
 
 /* Callbacks */
 
+static gboolean
+idle_notify_insert (gpointer data)
+{
+  GailEntry *entry = GAIL_ENTRY (data);
+
+  gail_entry_notify_insert (entry);
+  return FALSE;
+}
+
+static void
+gail_entry_notify_insert (GailEntry *entry)
+{
+  if (entry->signal_name)
+    {
+      g_signal_emit_by_name (entry, 
+                             entry->signal_name,
+                             entry->position,
+                             entry->length);
+      entry->signal_name = NULL;
+    }
+}
 /* Note arg1 returns the character at the start of the insert.
  * arg2 returns the number of characters inserted.
  */
-static void _gail_entry_insert_text_cb (GtkEntry *entry, 
-                                        gchar    *arg1, 
-                                        gint     arg2,
-                                        gpointer arg3)
+static void 
+_gail_entry_insert_text_cb (GtkEntry *entry, 
+                            gchar    *arg1, 
+                            gint     arg2,
+                            gpointer arg3)
 {
   AtkObject *accessible;
   GailEntry *gail_entry;
@@ -1083,7 +1094,9 @@ static void _gail_entry_insert_text_cb (GtkEntry *entry,
     }
   /*
    * The signal will be emitted when the cursor position is updated.
+   * or in an idle handler if it not updated.
    */
+   g_idle_add (idle_notify_insert, gail_entry);
 }
 
 static gunichar 
@@ -1121,9 +1134,10 @@ gail_entry_get_character_at_offset (AtkText *text,
 /* Note arg1 returns the start of the delete range, arg2 returns the
  * end of the delete range if multiple characters are deleted.	
  */
-static void _gail_entry_delete_text_cb (GtkEntry *entry, 
-                                        gint      arg1, 
-                                        gint      arg2)
+static void 
+_gail_entry_delete_text_cb (GtkEntry *entry, 
+                            gint      arg1, 
+                            gint      arg2)
 {
   AtkObject *accessible;
 
@@ -1218,7 +1232,7 @@ gail_entry_do_action (AtkAction *action,
       if (entry->action_idle_handler)
         return_value = FALSE;
       else
-        entry->action_idle_handler = gtk_idle_add (idle_do_action, entry);
+        entry->action_idle_handler = g_idle_add (idle_do_action, entry);
       break;
     default:
       return_value = FALSE;
