@@ -1,5 +1,5 @@
 /* GAIL - The GNOME Accessibility Implementation Library
- * Copyright 2001 Sun Microsystems Inc.
+ * Copyright 2001, 2002, 2003 Sun Microsystems Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -144,6 +144,13 @@ gail_menu_item_real_initialize (AtkObject *obj,
     }
   g_object_set_data (G_OBJECT (obj), "atk-component-layer",
                      GINT_TO_POINTER (ATK_LAYER_POPUP));
+
+  if (GTK_IS_TEAROFF_MENU_ITEM (data))
+    obj->role = ATK_ROLE_TEAR_OFF_MENU_ITEM;
+  else if (GTK_IS_SEPARATOR_MENU_ITEM (data))
+    obj->role = ATK_ROLE_SEPARATOR;
+  else
+    obj->role = ATK_ROLE_MENU_ITEM;
 }
 
 static void
@@ -169,13 +176,6 @@ gail_menu_item_new (GtkWidget *widget)
   accessible = ATK_OBJECT (object);
   atk_object_initialize (accessible, widget);
 
-  if (GTK_IS_TEAROFF_MENU_ITEM (widget))
-    accessible->role = ATK_ROLE_TEAR_OFF_MENU_ITEM;
-  else if (GTK_IS_SEPARATOR_MENU_ITEM (widget))
-    accessible->role = ATK_ROLE_SEPARATOR;
-  else
-    accessible->role = ATK_ROLE_MENU_ITEM;
-
   return accessible;
 }
 
@@ -280,7 +280,7 @@ gail_menu_item_do_action (AtkAction *action,
       if (gail_menu_item->action_idle_handler)
         return FALSE;
       else
-        gail_menu_item->action_idle_handler = gtk_idle_add (idle_do_action, gail_menu_item);
+        gail_menu_item->action_idle_handler = g_idle_add (idle_do_action, gail_menu_item);
       return TRUE;
     }
   else
@@ -533,7 +533,7 @@ gail_menu_item_finalize (GObject *object)
   g_free (menu_item->click_description);
   if (menu_item->action_idle_handler)
     {
-      gtk_idle_remove (menu_item->action_idle_handler);
+      g_source_remove (menu_item->action_idle_handler);
       menu_item->action_idle_handler = 0;
     }
 
