@@ -380,7 +380,22 @@ gail_notebook_page_ref_state_set (AtkObject *accessible)
     }
   else
     {
+      AtkObject *child;
+
+      child = atk_object_ref_accessible_child (accessible, 0);
+      g_return_val_if_fail (child, state_set);
+
       merged_state_set = state_set;
+      state_set = atk_object_ref_state_set (child);
+      if (atk_state_set_contains_state (state_set, ATK_STATE_VISIBLE))
+        {
+          atk_state_set_add_state (merged_state_set, ATK_STATE_VISIBLE);
+          if (atk_state_set_contains_state (state_set, ATK_STATE_SHOWING))
+              atk_state_set_add_state (merged_state_set, ATK_STATE_SHOWING);
+
+        } 
+      g_object_unref (state_set);
+      g_object_unref (child);
     }
   return merged_state_set;
 }
@@ -440,6 +455,7 @@ gail_notebook_page_get_extents (AtkComponent *component,
       g_return_if_fail (child);
 
       atk_component_get_position (ATK_COMPONENT (child), x, y, coord_type);
+      g_object_unref (child);
     }
   else
     {
