@@ -1,5 +1,5 @@
 /* GAIL - The GNOME Accessibility Enabling Library
- * Copyright 2001 Sun Microsystems Inc.
+ * Copyright 2001, 2002, 2003 Sun Microsystems Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -89,8 +89,6 @@ gail_scrolled_window_new (GtkWidget *widget)
   accessible = ATK_OBJECT (object);
   atk_object_initialize (accessible, widget);
 
-  accessible->role = ATK_ROLE_SCROLL_PANE;
-
   return accessible;
 }
 
@@ -109,6 +107,8 @@ gail_scrolled_window_real_initialize (AtkObject *obj,
   g_signal_connect_data (window->vscrollbar, "notify::visible",
     (GCallback)gail_scrolled_window_scrollbar_visibility_changed, 
     obj, FALSE, FALSE);
+
+  obj->role = ATK_ROLE_SCROLL_PANE;
 }
 
 static gint
