@@ -1,5 +1,5 @@
 /* GAIL - The GNOME Accessibility Implementation Library
- * Copyright 2001 Sun Microsystems Inc.
+ * Copyright 2001, 2002, 2003 Sun Microsystems Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -265,6 +265,9 @@ gail_text_view_real_initialize (AtkObject *obj,
      (GCallback) _gail_text_view_mark_set_cb, view, NULL, 0);
   g_signal_connect_data (buffer, "changed",
      (GCallback) _gail_text_view_changed_cb, view, NULL, 0);
+
+  obj->role = ATK_ROLE_TEXT;
+
 }
 
 static void
@@ -274,7 +277,7 @@ gail_text_view_finalize (GObject            *object)
 
   g_object_unref (text_view->textutil);
   if (text_view->insert_notify_handler)
-    gtk_idle_remove (text_view->insert_notify_handler);
+    g_source_remove (text_view->insert_notify_handler);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -292,8 +295,6 @@ gail_text_view_new (GtkWidget *widget)
   accessible = ATK_OBJECT (object);
   atk_object_initialize (accessible, widget);
 
-  accessible->role = ATK_ROLE_TEXT;
-
   return accessible;
 }
 
@@ -464,12 +465,11 @@ gail_text_view_get_character_at_offset (AtkText *text,
   if (offset >= gtk_text_buffer_get_char_count (buffer))
     return '\0';
 
-  gtk_text_buffer_get_start_iter (buffer, &start);
-  gtk_text_buffer_get_end_iter (buffer, &end);
-  string = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
-  index = g_utf8_offset_to_pointer (string, offset);
-
-  unichar = g_utf8_get_char (index);
+  gtk_text_buffer_get_iter_at_offset (buffer, &start, offset);
+  end = start;
+  gtk_text_iter_forward_char (&end);
+  string = gtk_text_buffer_get_slice (buffer, &start, &end, FALSE);
+  unichar = g_utf8_get_char (string);
   g_free(string);
   return unichar;
 }
@@ -579,6 +579,13 @@ gail_text_view_get_offset_at_point (AtkText      *text,
   buff_y = CLAMP (buff_y, rect.y, rect.y + rect.height - 1);
 
   gtk_text_view_get_iter_at_location (view, &loc_itr, buff_x, buff_y);
+  /*
+   * The iter at a location sometimes points to the next character.
+   * See bug 111031. We work around that
+   */
+  gtk_text_view_get_iter_location (view, &loc_itr, &rect);
+  if (buff_x < rect.x)
+    gtk_text_iter_backward_char (&loc_itr);
   return gtk_text_iter_get_offset (&loc_itr);
 }
 
@@ -866,9 +873,9 @@ gail_text_view_add_selection (AtkText *text,
   if (select_start == select_end)
     {
       gtk_text_buffer_get_iter_at_offset (buffer,  &pos_itr, start_pos);
-      gtk_text_buffer_move_mark_by_name (buffer, "insert", &pos_itr);
-      gtk_text_buffer_get_iter_at_offset (buffer,  &pos_itr, end_pos);
       gtk_text_buffer_move_mark_by_name (buffer, "selection_bound", &pos_itr);
+      gtk_text_buffer_get_iter_at_offset (buffer,  &pos_itr, end_pos);
+      gtk_text_buffer_move_mark_by_name (buffer, "insert", &pos_itr);
       return TRUE;
     }
   else
@@ -909,7 +916,6 @@ gail_text_view_remove_selection (AtkText *text,
       */
       cursor_mark = gtk_text_buffer_get_insert (buffer);
       gtk_text_buffer_get_iter_at_mark (buffer, &cursor_itr, cursor_mark);
-      gtk_text_buffer_move_mark_by_name (buffer, "insert", &cursor_itr);
       gtk_text_buffer_move_mark_by_name (buffer, "selection_bound", &cursor_itr);
       return TRUE;
     }
@@ -953,9 +959,9 @@ gail_text_view_set_selection (AtkText *text,
   if (select_start != select_end)
     {
       gtk_text_buffer_get_iter_at_offset (buffer,  &pos_itr, start_pos);
-      gtk_text_buffer_move_mark_by_name (buffer, "insert", &pos_itr);
-      gtk_text_buffer_get_iter_at_offset (buffer,  &pos_itr, end_pos);
       gtk_text_buffer_move_mark_by_name (buffer, "selection_bound", &pos_itr);
+      gtk_text_buffer_get_iter_at_offset (buffer,  &pos_itr, end_pos);
+      gtk_text_buffer_move_mark_by_name (buffer, "insert", &pos_itr);
       return TRUE;
     }
   else
@@ -1415,7 +1421,7 @@ _gail_text_view_insert_text_cb (GtkTextBuffer *buffer,
        */
       if (gail_text_view->insert_notify_handler)
         {
-          gtk_idle_remove (gail_text_view->insert_notify_handler);
+          g_source_remove (gail_text_view->insert_notify_handler);
         }
       gail_text_view->insert_notify_handler = 0;
       insert_idle_handler (gail_text_view);
@@ -1449,7 +1455,7 @@ _gail_text_view_delete_range_cb (GtkTextBuffer *buffer,
   gail_text_view = GAIL_TEXT_VIEW (accessible);
   if (gail_text_view->insert_notify_handler)
     {
-      gtk_idle_remove (gail_text_view->insert_notify_handler);
+      g_source_remove (gail_text_view->insert_notify_handler);
       gail_text_view->insert_notify_handler = 0;
       if (gail_text_view->position == offset && 
           gail_text_view->length == length)
@@ -1544,7 +1550,7 @@ _gail_text_view_changed_cb (GtkTextBuffer *buffer,
     {
       if (!gail_text_view->insert_notify_handler)
         {
-          gail_text_view->insert_notify_handler = gtk_idle_add (insert_idle_handler, accessible);
+          gail_text_view->insert_notify_handler = g_idle_add (insert_idle_handler, accessible);
         }
       return;
     }
@@ -1609,7 +1615,7 @@ emit_text_caret_moved (GailTextView *gail_text_view,
    */
   if (gail_text_view->insert_notify_handler)
     {
-      gtk_idle_remove (gail_text_view->insert_notify_handler);
+      g_source_remove (gail_text_view->insert_notify_handler);
       gail_text_view->insert_notify_handler = 0;
       insert_idle_handler (gail_text_view);
     }
