@@ -281,7 +281,7 @@ gail_toplevel_hide_event_watcher (GSignalInvocationHint *ihint,
   object = g_value_get_object (param_values + 0);
 
   if (!GTK_IS_WINDOW (object))
-    return FALSE;
+    return TRUE;
 
   _gail_toplevel_remove_child (toplevel, GTK_WINDOW (object));
   return TRUE;
@@ -349,9 +349,13 @@ is_attached_menu_window (GtkWidget *widget)
       GtkWidget *attach;
 
       attach = gtk_menu_get_attach_widget (GTK_MENU (child));
+      /* Allow for menu belonging to the Panel Menu, which is a GtkButton */
       if (GTK_IS_MENU_ITEM (attach) ||
-          GTK_IS_OPTION_MENU (attach))
+          GTK_IS_OPTION_MENU (attach) ||
+          GTK_IS_BUTTON (attach))
         ret = TRUE;
+      else if (attach)
+        g_warning ("Unexpected attach type %s\n", G_OBJECT_TYPE_NAME (attach));
     }
   return ret;
 }
