@@ -230,11 +230,14 @@ atk_key_event_from_gdk_event_key (GdkEventKey *key)
   event->state = key->state;
   event->keyval = key->keyval;
   event->length = key->length;
-  if (key->string && key->string [0] && g_unichar_isgraph (g_utf8_get_char (key->string)))
+  if (key->string && key->string [0] && 
+      (key->state & GDK_CONTROL_MASK ||
+       g_unichar_isgraph (g_utf8_get_char (key->string))))
     {
       event->string = key->string;
     }
-  else if (key->type == GDK_KEY_PRESS)
+  else if (key->type == GDK_KEY_PRESS ||
+           key->type == GDK_KEY_RELEASE)
     {
       event->string = gdk_keyval_name (key->keyval);	    
     }
@@ -481,8 +484,14 @@ window_removed (AtkObject *atk_obj,
                  guint     index,
                  AtkObject *child)
 {
+  GtkWidget *widget;
+
   g_return_if_fail (GAIL_IS_WINDOW (child));
 
+  widget = GTK_ACCESSIBLE (child)->widget;
+  g_return_if_fail (widget);
+
+  g_signal_handlers_disconnect_by_func (widget, (GCallback) window_focus, NULL);
   g_signal_emit (child, g_signal_lookup ("destroy", GAIL_TYPE_WINDOW), 0); 
 }
 
