@@ -344,7 +344,8 @@ gail_widget_get_parent (AtkObject *accessible)
 }
 
 static GtkWidget*
-find_label (GtkWidget *widget)
+find_label (GtkWidget *start,
+            GtkWidget *widget)
 {
 
   GList *list, *l;
@@ -352,7 +353,7 @@ find_label (GtkWidget *widget)
   GtkWidget *label;
 
   label = NULL;
-  parent = gtk_widget_get_parent (widget);
+  parent = gtk_widget_get_parent (start);
   if (GTK_IS_CONTAINER (parent))
     {
       list = gtk_container_get_children (GTK_CONTAINER (parent));
@@ -362,7 +363,7 @@ find_label (GtkWidget *widget)
         {
           while (l)
             {
-              if ((GtkWidget *)l->data == widget)
+              if ((GtkWidget *)l->data == start)
                 break;
 
               l = l->next;
@@ -373,11 +374,17 @@ find_label (GtkWidget *widget)
            */
           g_return_val_if_fail (l, NULL);
           if (l->prev && GTK_IS_LABEL (l->prev->data))
-            label = GTK_WIDGET (l->prev->data);
-          else if (l->next && GTK_IS_LABEL (l->next->data))
-            label = GTK_WIDGET (l->next->data);
-          else
-            label = NULL;
+	    {  
+              label = GTK_WIDGET (l->prev->data);
+              if (gtk_label_get_mnemonic_widget (GTK_LABEL (label)) != widget)
+                label = NULL;
+            } 
+          if (!label && l->next && GTK_IS_LABEL (l->next->data))
+	    {  
+              label = GTK_WIDGET (l->next->data);
+              if (gtk_label_get_mnemonic_widget (GTK_LABEL (label)) != widget)
+                label = NULL;
+            } 
 
           g_list_free (list);
        }
@@ -417,25 +424,22 @@ gail_widget_ref_relation_set (AtkObject *obj)
        * If this fails try to find a label at the same level as the widget's
        * parent. This finds labels for NautilusProgramChooser dialog.
        */
-      label = find_label (widget);
+      label = find_label (widget, widget);
       if (!label) 
         {
           GtkWidget *parent;
 
           parent = gtk_widget_get_parent (widget);
           if (parent)
-            label = find_label (parent);
+            label = find_label (parent, widget);
         }
       if (label)
         {
-          if (gtk_label_get_mnemonic_widget (GTK_LABEL (label)) == widget)
-            {
-	      array [0] = gtk_widget_get_accessible (label);
+	  array [0] = gtk_widget_get_accessible (label);
 
-	      relation = atk_relation_new (array, 1, ATK_RELATION_LABELLED_BY);
-	      atk_relation_set_add (relation_set, relation);
-	      g_object_unref (relation);
-            }
+	  relation = atk_relation_new (array, 1, ATK_RELATION_LABELLED_BY);
+	  atk_relation_set_add (relation_set, relation);
+	  g_object_unref (relation);
         }
     }
 
