@@ -38,6 +38,7 @@ static void gail_window_class_init (GailWindowClass *klass);
 
 static void                  gail_window_real_initialize (AtkObject    *obj,
                                                           gpointer     data);
+static void                  gail_window_finalize        (GObject      *object);
 
 static G_CONST_RETURN gchar* gail_window_get_name       (AtkObject     *accessible);
 
@@ -112,8 +113,11 @@ static void
 gail_window_class_init (GailWindowClass *klass)
 {
   GailWidgetClass *widget_class;
+  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   AtkObjectClass  *class = ATK_OBJECT_CLASS (klass);
 
+  gobject_class->finalize = gail_window_finalize;
+
   widget_class = (GailWidgetClass*)klass;
   widget_class->focus_gtk = gail_window_real_focus_gtk;
   widget_class->notify_gtk = gail_window_real_notify_gtk;
@@ -229,7 +233,7 @@ gail_window_real_initialize (AtkObject *obj,
 
   ATK_OBJECT_CLASS (parent_class)->initialize (obj, data);
 
-
+  GAIL_WINDOW (obj)->name_change_handler = 0;
   widget = GTK_WIDGET (data);
 
   g_signal_connect (data,
@@ -265,6 +269,20 @@ gail_window_real_initialize (AtkObject *obj,
     obj->role = ATK_ROLE_INVALID;
 }
 
+static void
+gail_window_finalize (GObject *object)
+{
+  GailWindow* window = GAIL_WINDOW (object);
+
+  if (window->name_change_handler)
+    {
+      g_source_remove (window->name_change_handler);
+      window->name_change_handler = 0;
+    }
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
 static G_CONST_RETURN gchar*
 gail_window_get_name (AtkObject *accessible)
 {
@@ -395,23 +413,41 @@ gail_window_ref_state_set (AtkObject *accessible)
   return state_set;
 }
 
+static gboolean
+idle_notify_name_change (gpointer data)
+{
+  GailWindow *window;
+  AtkObject *obj;
+
+  window = GAIL_WINDOW (data);
+  window->name_change_handler = 0;
+  if (GTK_ACCESSIBLE (window)->widget == NULL)
+    return FALSE;
+
+  obj = ATK_OBJECT (window);
+  if (obj->name == NULL)
+    {
+    /*
+     * The title has changed so notify a change in accessible-name
+     */
+      g_object_notify (G_OBJECT (obj), "accessible-name");
+    }
+  g_signal_emit_by_name (obj, "visible_data_changed");
+  return FALSE;
+}
+
 static void
 gail_window_real_notify_gtk (GObject		*obj,
                              GParamSpec		*pspec)
 {
   GtkWidget *widget = GTK_WIDGET (obj);
   AtkObject* atk_obj = gtk_widget_get_accessible (widget);
+  GailWindow *window = GAIL_WINDOW (atk_obj);
 
   if (strcmp (pspec->name, "title") == 0)
     {
-      if (atk_obj->name == NULL)
-        {
-        /*
-         * The title has changed so notify a change in accessible-name
-         */
-          g_object_notify (G_OBJECT (atk_obj), "accessible-name");
-	}
-      g_signal_emit_by_name (atk_obj, "visible_data_changed");
+      if (window->name_change_handler == 0)
+        window->name_change_handler = g_idle_add (idle_notify_name_change, atk_obj);    
     }
   else
     GAIL_WIDGET_CLASS (parent_class)->notify_gtk (obj, pspec);
